==> skypager-project-2.33.0.tgz: package/src/helpers/definitions/action.js <==
import { any, assign, noConflict, tabelize, parameterize, singularize, underscore } from '../../util'


const route = require('path-match')({
  sensitive: false,
  strict: false,
  end: false
})

let tracker = { }
let _curr

function current () { return tracker[_curr] }
function clearDefinition () { _curr = null; delete tracker[_curr] }

/**
 * An ActionDefinition is created automatically when an action file is loaded
 * by the actions registry.  A standard action file starts with:
 *
 * @example
 *
 *  action('MyAction')
 *
 *  execute(function(params = {}, context = {project}){
 *
 *  })
 */
export class ActionDefinition {
  constructor (actionName) {
    this.name = actionName
    this.description = ''
    this.config = {}
    this.interfaces = {}
    this.parameters = { }
    this.aliases = {}
    this.validator = function () { return true }
    this.executor = function () { throw ('Define your own executor function') }
    this.config.pathMatchers = []
  }

  describe(value) {
    this.description = value
  }

  expose (platform, configurator) {
    this.interfaces[platform] = configurator
  }

  addCommandPhrase(phrase){
    this.config.pathMatchers.push(
      route((phrase.trim()).replace(/\s/g, '/'))
    )
  }

  addRouteMatcher(rule){
    this.config.pathMatchers.push(
      route(rule)
    )
  }

  testRoute(path){
   let matching = this.config.pathMatchers.find(rule => rule(path))
   return matching && matching(path)
  }

  testCommand(phrase){
    let sample = trim(phrase).replace(/\s/g, '/')
    let matching = this.config.pathMatchers.find(rule => rule(sample))
    return matching && matching(sample)
  }

  get api () {
    let action = this

    return {
      name: this.name,
      aliases: this.aliases,
      testCommand: this.testCommand.bind(this),
      testRoute: this.testRoute.bind(this),
      execute: this.executor,
      validate: this.validator,
      parameters: this.parameters,
      runner: function (...args) {
        let report = {
          errors:[],
          suggestions: [],
          warnings:[]
        }

        let _context = args[ args.length - 1 ]
        let logError = console.error
        let logInfo = console.log
        let logWarn = console.warn

        if (_context.project) {
          logError = _context.project.error
          logInfo = _context.project.info
          logWarn = _context.project.warn
        }

        let context = {
           ...(_context),
           report
        }

        let localHelpers = {
           abort(message, ...args) {
             logError(message, ...args)
             report.errors.push(message, ...args)
             process.exit(1)
            },
            error(message, ...args) {
              action._helper.emit('error', message, ...args)
              logError(message, ...args)
              report.errors.push(message)
            },
            warn(message, ...args) {
              action._helper.emit('warn', message, ...args)
              logWarn(message, ...args)
              report.warnings.push(message)
            },
            //deprecated
            suggest(message, ...args) {
              action._helper.emit('suggest', message, ...args)
              logInfo(message, ...args)
              report.suggestions.push(message)
            },
            info(message, ...args) {
              action._helper.emit('info', message, ...args)
              logInfo(message, ...args)
              report.suggestions.push(message)
            },
            done(result, ...args) {
              logInfo(result, ...args)
              action._helper.emit('done', result, ...args)
            },
            report
        }

        let passesValidation = noConflict(function(){
          return action.api.validate(...args)
        }, localHelpers)(...args)

        if (passesValidation === false) {
          report.success = false
          return report
        }

        report.result = noConflict(function(){
          try {
            let r = action.api.execute(...args)

            if (r) {
              report.success = true
              report.result = r
            }
          } catch(err) {
            report.errors.push('fatal error:' + err.message)
            report.errors.push(err.stack)
            throw(err)
          }

          return report
        }, localHelpers)(...args)

        return report
      }
    }
  }

  aka (...list) {
    list.forEach(alias => {
      this.aliases[ parameterize(underscore(alias.toLowerCase())) ] = true
    })
  }

  aliases (...list) {
    this.aka(...list)
  }

  pathMatchers(...args) {

  }

  params (...args) {
    this.configureParameters(...args)
  }

  validate (fn) {
    this.validator = fn
  }

  execute (fn) {
    this.executor = fn
  }

  configureParameters(inputs) {

  }
}

ActionDefinition.current = current
ActionDefinition.clearDefinition = clearDefinition

export function DSL (fn) {
  noConflict(fn, DSL)()
}

export function lookup(actionName) {
  return tracker[(_curr = tabelize(parameterize(actionName)).toLowerCase())]
}

assign(DSL, {
  action: function (actionName) {
    tracker[(_curr = tabelize(parameterize(actionName)))] = new ActionDefinition(actionName)
  },
  describe: function (...args) { tracker[_curr].describe(...args) },
  aliases: function (...args) { tracker[_curr].aliases(...args) },
  aka: function (...args) { tracker[_curr].aka(...args) },
  validate: function (...args) { tracker[_curr].validate(...args) },
  execute: function (...args) { tracker[_curr].execute(...args) },
  required: function (...args) { tracker[_curr].required(...args) },
  optional: function (...args) { tracker[_curr].optional(...args) },
  params: function (...args) { tracker[_curr].params(...args) },
  commandPhrase: function (...args) { tracker[_curr].addCommandPhrase(...args) },
  route: function (...args) { tracker[_curr].addRouteMatcher(...args) },
  expose: function(...args) { tracker[_curr].expose(...args) },
  cli: function(...args) { tracker[_curr].expose('cli', ...args) },
  ipc: function(...args) { tracker[_curr].expose('ipc', ...args) },
  web: function(...args) { tracker[_curr].expose('web', ...args) }
})

==> skypager-project-2.55.2.tgz: package/src/helpers/definitions/action.js <==
import { any, assign, noConflict, tableize, parameterize, singularize, underscore } from '../../util'


const route = require('path-match')({
  sensitive: false,
  strict: false,
  end: false
})

let tracker = { }
let _curr

function current () { return tracker[_curr] }
function clearDefinition () { _curr = null; delete tracker[_curr] }

/**
 * An ActionDefinition is created automatically when an action file is loaded
 * by the actions registry.  A standard action file starts with:
 *
 * @example
 *
 *  action('MyAction')
 *
 *  execute(function(params = {}, context = {project}){
 *
 *  })
 */
export class ActionDefinition {
  constructor (actionName) {
    this.name = actionName
    this.description = ''
    this.config = {}
    this.interfaces = {}
    this.parameters = { }
    this.aliases = {}
    this.validator = function () { return true }
    this.executor = function () { throw ('Define your own executor function') }
    this.config.pathMatchers = []
  }

  describe(value) {
    this.description = value
  }

  expose (platform, configurator) {
    this.interfaces[platform] = configurator
  }

  addCommandPhrase(phrase){
    this.config.pathMatchers.push(
      route((phrase.trim()).replace(/\s/g, '/'))
    )
  }

  addRouteMatcher(rule){
    this.config.pathMatchers.push(
      route(rule)
    )
  }

  testRoute(path){
   let matching = this.config.pathMatchers.find(rule => rule(path))
   return matching && matching(path)
  }

  testCommand(phrase){
    let sample = trim(phrase).replace(/\s/g, '/')
    let matching = this.config.pathMatchers.find(rule => rule(sample))
    return matching && matching(sample)
  }

  get api () {
    let action = this

    return {
      name: this.name,
      aliases: this.aliases,
      testCommand: this.testCommand.bind(this),
      testRoute: this.testRoute.bind(this),
      execute: this.executor,
      validate: this.validator,
      parameters: this.parameters,


      /**
      * This really belongs on the action.
      *
      * The purpose of this function is to
      *
      *   1) ensure all calls to the actions execute fn
      *      first pass through the validation function
      *
      *   2) ensure that all calls to execute pass predictable
      *      context args as the last argument
      *
      *   3) ensure that a consistent result object is returned
      *
      **/
      runner: function (...args) {
        let report = {
          errors:[],
          suggestions: [],
          warnings:[]
        }

        let _context = args[ args.length - 1 ]
        let logError = console.error
        let logInfo = console.log
        let logWarn = console.warn

        if (_context.project && _context.project) {
          logError = _context.project.error
          logInfo = _context.project.info
          logWarn = _context.project.warn
        }

        let context = {
           ...(_context),
           report
        }

        let localHelpers = {
           abort(message, ...args) {
             logError(message, ...args)
             report.success = false
             report.errors.push(message, ...args)
             process.exit(1)
            },
            error(message, ...args) {
              action._helper.emit('error', message, ...args)
              logError(message, ...args)
              report.success = false
              report.errors.push(message)
            },
            warn(message, ...args) {
              action._helper.emit('warn', message, ...args)
              logWarn(message, ...args)
              report.warnings.push(message)
            },
            //deprecated
            suggest(message, ...args) {
              action._helper.emit('suggest', message, ...args)
              logInfo(message, ...args)
              report.suggestions.push(message)
            },
            info(message, ...args) {
              action._helper.emit('info', message, ...args)
              logInfo(message, ...args)
              report.suggestions.push(message)
            },
            done(result, ...args) {
              logInfo(result, ...args)
              report.success = true
              action._helper.emit('done', result, ...args)
            },
            report
        }

        let passesValidation = noConflict(function(){
          return action.api.validate(...args)
        }, localHelpers)(...args)

        if (passesValidation === false) {
          report.success = false
          return report
        }

        report.result = noConflict(function(){
          try {
            let r = action.api.execute(...args)

            if (r) {
              report.result = r
            }
          } catch(err) {
            report.success = false
            report.errors.push('fatal error:' + err.message)
            report.errors.push(err.stack)
            throw(err)
          }

          return report

        }, localHelpers)(...args)

        return report
      }
    }
  }

  aka (...list) {
    list.forEach(alias => {
      this.aliases[ parameterize(underscore(alias.toLowerCase())) ] = true
    })
  }

  aliases (...list) {
    this.aka(...list)
  }

  pathMatchers(...args) {

  }

  params (...args) {
    this.configureParameters(...args)
  }

  validate (fn) {
    this.validator = fn
  }

  execute (fn) {
    this.executor = fn
  }

  configureParameters(inputs) {

  }
}

ActionDefinition.current = current
ActionDefinition.clearDefinition = clearDefinition

export function DSL (fn) {
  noConflict(fn, DSL)()
}

export function lookup(actionName) {
  return tracker[(_curr = tableize(parameterize(actionName)).toLowerCase())]
}

assign(DSL, {
  action: function (actionName) {
    tracker[(_curr = tableize(parameterize(actionName)))] = new ActionDefinition(actionName)
  },
  describe: function (...args) { tracker[_curr].describe(...args) },
  aliases: function (...args) { tracker[_curr].aliases(...args) },
  aka: function (...args) { tracker[_curr].aka(...args) },
  validate: function (...args) { tracker[_curr].validate(...args) },
  execute: function (...args) { tracker[_curr].execute(...args) },
  required: function (...args) { tracker[_curr].required(...args) },
  optional: function (...args) { tracker[_curr].optional(...args) },
  params: function (...args) { tracker[_curr].params(...args) },
  commandPhrase: function (...args) { tracker[_curr].addCommandPhrase(...args) },
  route: function (...args) { tracker[_curr].addRouteMatcher(...args) },
  expose: function(...args) { tracker[_curr].expose(...args) },
  cli: function(...args) { tracker[_curr].expose('cli', ...args) },
  ipc: function(...args) { tracker[_curr].expose('ipc', ...args) },
  endpoint: function(...args) { tracker[_curr].expose('endpoint', ...args) }
})
